@@ -10,6 +10,7 @@
 #include "../distances/calculators.h"
 #include "../distances/details.h" // sdtw
 #include "../utils/KahanSummer.h"
+#include "../utils/ParallelWorker.h"
 #include "../utils/SurrogateMatrix.h"
 #include "../utils/TSTSList.h"
 #include "../utils/utils.h" // get_grain, id_t
@@ -107,15 +108,17 @@ public:
 /* the parallel worker for the univariate version */
 // =================================================================================================
 
-class SdtwUv : public RcppParallel::Worker {
+class SdtwUv : public ParallelWorker {
 public:
     // constructor
     SdtwUv(const SdtwCentCalculator&& dist_calculator,
            const Rcpp::NumericVector& weights,
            Rcpp::NumericVector& gradient,
            double& objective,
-           const double gamma)
-        : dist_calculator_(dist_calculator)
+           const double gamma,
+           const int grain)
+        : ParallelWorker(grain, 1, 10)
+        , dist_calculator_(dist_calculator)
         , weights_(weights)
         , gamma_(gamma)
         , gradient_summer_(&gradient[0], gradient.length())
@@ -123,12 +126,13 @@ public:
     { }
 
     // parallel loop across specified range
-    void operator()(std::size_t begin, std::size_t end) {
+    void work_it(std::size_t begin, std::size_t end) override {
         // local copy of calculator so it is setup separately for each thread
         mutex_.lock();
         SdtwCentCalculator* local_calculator = dist_calculator_.clone();
         SurrogateMatrix<double> em(2, local_calculator->max_len_y_ + 2);
         mutex_.unlock();
+
         // calculations for these series
         //   Includes the calculation of soft-DTW gradient + jacobian product.
         SurrogateMatrix<double>& cm = local_calculator->cm_;
@@ -136,6 +140,8 @@ public:
         const auto& x = local_calculator->x_[0];
         id_t m = x.n_rows;
         for (std::size_t id = begin; id < end; id++) {
+            if (is_interrupted(id)) break; // nocov
+
             const auto& y = local_calculator->y_[id];
             double dist = local_calculator->calculate(0,id);
 
@@ -157,6 +163,7 @@ public:
                 if (i == m) em((m+1)%2,n+1) = 0;
             }
         }
+
         // finish
         mutex_.lock();
         delete local_calculator;
@@ -168,22 +175,23 @@ private:
     const RcppParallel::RVector<double> weights_;
     double gamma_;
     KahanSummer gradient_summer_, objective_summer_;
-    tthread::mutex mutex_;
 };
 
 // =================================================================================================
 /* the parallel worker for the multivariate version */
 // =================================================================================================
 
-class SdtwMv : public RcppParallel::Worker {
+class SdtwMv : public ParallelWorker {
 public:
     // constructor
     SdtwMv(const SdtwCentCalculator&& dist_calculator,
            const Rcpp::NumericVector& weights,
            Rcpp::NumericMatrix& gradient,
            double& objective,
-           const double gamma)
-        : dist_calculator_(dist_calculator)
+           const double gamma,
+           const int grain)
+        : ParallelWorker(grain, 1, 10)
+        , dist_calculator_(dist_calculator)
         , weights_(weights)
         , gamma_(gamma)
         , gradient_summer_(&gradient[0], gradient.nrow(), gradient.ncol())
@@ -191,12 +199,13 @@ public:
     { }
 
     // parallel loop across specified range
-    void operator()(std::size_t begin, std::size_t end) {
+    void work_it(std::size_t begin, std::size_t end) override {
         // local copy of calculator so it is setup separately for each thread
         mutex_.lock();
         SdtwCentCalculator* local_calculator = dist_calculator_.clone();
         SurrogateMatrix<double> em(2, local_calculator->max_len_y_ + 2);
         mutex_.unlock();
+
         // calculations for these series
         //   Includes the calculation of soft-DTW gradient + jacobian product.
         double* grad = nullptr;
@@ -205,6 +214,8 @@ public:
         const auto& x = local_calculator->x_[0];
         id_t m = x.n_rows, dim = x.n_cols;
         for (std::size_t id = begin; id < end; id++) {
+            if (is_interrupted(id)) break; // nocov
+
             const auto& y = local_calculator->y_[id];
             double dist = local_calculator->calculate(0,id);
 
@@ -232,6 +243,7 @@ public:
                 if (i == m) em((m+1)%2, n+1) = 0;
             }
         }
+
         // finish
         mutex_.lock();
         delete local_calculator;
@@ -244,7 +256,6 @@ private:
     const RcppParallel::RVector<double> weights_;
     double gamma_;
     KahanSummer gradient_summer_, objective_summer_;
-    tthread::mutex mutex_;
 };
 
 // =================================================================================================
@@ -261,15 +272,19 @@ extern "C" SEXP sdtw_cent(SEXP SERIES, SEXP CENTROID,
     int num_threads = Rcpp::as<int>(NUM_THREADS);
     int grain = get_grain(series.length(), num_threads);
     if (grain == DTWCLUST_MIN_GRAIN) grain = 1;
+
     // compute objective and gradient
     if (Rcpp::as<bool>(MV)) {
         Rcpp::NumericMatrix cent(CENTROID);
         Rcpp::NumericMatrix gradient(cent.nrow(), cent.ncol());
+
         double objective = 0;
         Rcpp::List x = Rcpp::List::create(CENTROID);
+
         SdtwCentCalculator dist_calculator(x, series, gamma);
-        SdtwMv parallel_worker(std::move(dist_calculator), WEIGHTS, gradient, objective, gamma);
-        RcppParallel::parallelFor(0, series.length(), parallel_worker, grain);
+        SdtwMv parallel_worker(std::move(dist_calculator), WEIGHTS, gradient, objective, gamma, grain);
+        parallel_for(0, series.length(), parallel_worker, grain);
+
         return Rcpp::List::create(
             Rcpp::_["objective"] = objective,
             Rcpp::_["gradient"] = gradient
@@ -278,11 +293,14 @@ extern "C" SEXP sdtw_cent(SEXP SERIES, SEXP CENTROID,
     else {
         Rcpp::NumericVector cent(CENTROID);
         Rcpp::NumericVector gradient(cent.length());
+
         double objective = 0;
         Rcpp::List x = Rcpp::List::create(CENTROID);
+
         SdtwCentCalculator dist_calculator(x, series, gamma);
-        SdtwUv parallel_worker(std::move(dist_calculator), WEIGHTS, gradient, objective, gamma);
-        RcppParallel::parallelFor(0, series.length(), parallel_worker, grain);
+        SdtwUv parallel_worker(std::move(dist_calculator), WEIGHTS, gradient, objective, gamma, grain);
+        parallel_for(0, series.length(), parallel_worker, grain);
+
         return Rcpp::List::create(
             Rcpp::_["objective"] = objective,
             Rcpp::_["gradient"] = gradient
